require.config({
    paths: {
        'react': '/module/easyBinding/rc/node_modules/react/dist/react-with-addons',
        'react-dom': '/module/easyBinding/rc/node_modules/react-dom/dist/react-dom',
        'moment': '/module/easyBinding/rc/node_modules/moment/moment',
        'myD3': '/module/easyBinding/rc/node_modules/d3/dist/d3.min',

        'underscore': '/module/easyBinding/rc/node_modules/underscore/underscore',
        'html2canvas': '/module/easyBinding/rc/node_modules/html2canvas/dist/html2canvas',
    }
});

define(
    [
        'jquery',
        'baja!',
        'bajaux/Widget',
        'bajaux/mixin/subscriberMixIn',
        'bajaux/dragdrop/dragDropUtils',
        'bajaScript/baja/ord/SlotPath',
        'react',
        'react-dom',
        'moment',
        'myD3',

        'underscore',
        'html2canvas',

        'nmodule/easyBinding/rc/Util/EbUtils',
        'nmodule/easyBinding/rc/widgetJS/js/EasyWidgetEdit',
        'nmodule/easyBinding/rc/widgetJS/js/PopupLeftPanelPointFilter',
        'nmodule/easyBinding/rc/widgetJS/js/TreeComponent',
        'nmodule/easyBinding/rc/widgetJS/js/PointChooserComponent',
        'nmodule/easyBinding/rc/widgetJS/js/EasyBinding',
        'css!nmodule/easyBinding/rc/EasyBinding/EasyBinding',
        'css!nmodule/webEditors/rc/fe/webEditors-structure'
    ],
    function ($,
              baja,
              Widget,
              subscriberMixIn,
              dragDropUtils,
              slotPath,
              react,
              reactDOM,
              moment,
              myD3,
              underscore,
              html2canvas,
              EbUtils,
              EasyWidgetEditFile,
              PopupLeftPanelPointFilter,
              SearchTreeFile,
              PointChooserFile,
              easyBindingFileRenderDOM
    ) {

        var EasyBinding = function () {
            var that = this;
            // This calls the super constructor.
            Widget.apply(that, arguments);

            // Applying a Subscriber MixIn adds subscription capabilities to a
            // Widget. Once added, the value passed into 'doLoad' is fully subscribed
            // and 'live'.
            subscriberMixIn(that);
        };
        var easyLogsToStationConsole = function (msg) {
            baja.rpc({
                typeSpec: 'easyBinding:EasyBindingUtil',
                method: 'logToStation',
                args: [msg]
            }).then(function () {
                //Success
            });

        };
        var easyBindinglogBajaInfo = function (info, isError) {
            if (isError) {
                baja.outln("Error:" + info);
            } else {
                baja.outln("Info:" + info);
            }
        }

        //extend and set up prototype chain
        EasyBinding.prototype = Object.create(Widget.prototype);
        EasyBinding.prototype.constructor = EasyBinding;

        EasyBinding.prototype.doInitialize = function (element) {
            element.addClass("d3_EasyBindingOuter");

            var EasyWidgetEdit = EasyWidgetEditFile(
                myD3,
                react,
                reactDOM,
                100,
                100,
                dragDropUtils,baja,easyBindinglogBajaInfo
            );
            var searchTree = SearchTreeFile(
                react,
                reactDOM,
                baja,
                easyBindinglogBajaInfo
            );
            var popupLeftPanelPointFilter = PopupLeftPanelPointFilter(react, reactDOM, baja, easyBindinglogBajaInfo);
            var pointChooser = PointChooserFile(
                react,
                reactDOM,
                baja,
                slotPath,
                easyBindinglogBajaInfo, popupLeftPanelPointFilter, searchTree
            );
            //Render chart
            this.RenderDOMResult = easyBindingFileRenderDOM($,
                baja,
                slotPath,
                myD3,
                react,
                reactDOM,
                underscore,
                html2canvas,
                EbUtils,
                EasyWidgetEdit,
                this.jq().height(),
                this.jq().width(),
                element[0], easyLogsToStationConsole, easyBindinglogBajaInfo, dragDropUtils, searchTree, pointChooser);
        };

        //A prop changes
        EasyBinding.prototype.doChanged = function (value) {
            var _This = this;
            _This.RenderDOMResult.updatedLayout();
        }

        EasyBinding.prototype.doLoad = function (value) {
            var _This = this;
            baja.rpc({
                typeSpec: 'easyBinding:EasyBindingUtil',
                method: 'checkFeatureLicense'
            })
                .then(function (returnValue) {
                    if (!returnValue) {
                        baja.rpc({
                            typeSpec: 'easyBinding:EasyBindingUtil',
                            method: 'checkEdgeController'
                        }).then(function (returnEdge) {
                            if (returnEdge) {
                                _This.RenderDOMResult.setPxContents("EdgeController");
                            } else {
                                _This.RenderDOMResult.setPxContents("NO_LICENSE");
                            }
                        });
                    } else {
                        //show feature not available message for the controller when license is available for the edge controller
                        baja.rpc({
                            typeSpec: 'easyBinding:EasyBindingUtil',
                            method: 'checkEdgeController'
                        }).then(function (returnEdge) {
                            if (returnEdge) {
                                _This.RenderDOMResult.setPxContents("EdgeController");
                            } else {
                                if (value.$fileData != undefined) {
                                    Promise.resolve($.ajax(value.$fileData.read + '?snoop=false', {dataType: "text"})).then(function (resp) {
                                        _This.RenderDOMResult.setPxContents(resp);
                                    });
                                    _This.RenderDOMResult.setPxOrd(value.$fileData.ordInSession);
                                } else {
                                    // we need to find if this component has a Px View on it - we'll take the first one thank you
                                    for (var agent in value.$map.$map) {
                                        try {
                                            if(value.$map.$map.hasOwnProperty(agent)) {
                                                if (value.$map.$map[agent].$display === "Px View") {
                                                    Promise.resolve($.ajax("/ord/" + value.$map.$map[agent].$val.getPxFile().$ord + '%7Cview:web:FileDownloadView?snoop=false', {dataType: "text"})).then(function (resp) {
                                                        _This.RenderDOMResult.setPxContents(resp);
                                                    }, function (err) {
                                                        _This.RenderDOMResult.cannotLoadView();
                                                    });
                                                    _This.RenderDOMResult.setPxOrd(value.$map.$map[agent].$val.getPxFile().$ord);
                                                    return;
                                                }
                                            }
                                        } catch (error) {
                                            _This.RenderDOMResult.cannotLoadView();
                                        }
                                    }
                                    _This.RenderDOMResult.cannotLoadView();
                                }
                            }
                        });
                    }
                });
        }


        //Called when ever the widget is resized
        EasyBinding.prototype.doLayout = function (value) {
            var _This = this;
            _This.RenderDOMResult.updatedLayout();
        };

        return EasyBinding;

    }
);
