/**
 * @copyright 2017 Tridium, Inc. All Rights Reserved.
 * @author Tony Richards
 *
 * @private
 * this is a copy from the CalendarScheduler.js with extensions: save to controller
 */
define(['baja!',
        'bajaux/events',
        'bajaux/mixin/responsiveMixIn',
        'bajaux/mixin/subscriberMixIn',
        'bajaux/util/SaveCommand',
        'nmodule/js/rc/log/Log', 'nmodule/schedule/rc/baja/fe/ActiveDatesCalendar',
        'nmodule/schedule/rc/fe/CalendarEventsEditor',
        'nmodule/schedule/rc/servlets/ssc',
        'nmodule/schedule/rc/util/scheduleUtils',
        'nmodule/webEditors/rc/fe/CompositeEditor',
        'nmodule/webEditors/rc/fe/config/CompositeBuilder', 'Promise'], function (baja,
                                                                                  events, responsiveMixIn,
                                                                                  subscribable,
                                                                                  SaveCommand,
                                                                                  Log,
                                                                                  ActiveDatesCalendar,
                                                                                  CalendarEventsEditor,
                                                                                  ssc,
                                                                                  scheduleUtils,
                                                                                  CompositeEditor,
                                                                                  CompositeBuilder)
{

  'use strict';

  var MODIFY_EVENT = events.MODIFY_EVENT;


  var tpl = function (ed) {
      return '\n    <div class=\"activeDates\"></div>\n    <div class=\"calendarScheduler\"></div>';
  };

 var _calendar= null;
  /**
   * Calendar Scheduler
   *
   * An editor that edits calendar events from a snapshot
   *
   * @class
   * @alias module:nmodule/schedule/rc/fe/CalendarScheduler
   * @extends module:nmodule/webEditors/rc/fe/CompositeEditor
   */
  var HonCalendarScheduler = function (params) {
    CompositeEditor.apply(this, arguments);
    this.getCommandGroup().add(new SaveCommand());

    subscribable(this);
      responsiveMixIn(this, {
          'calendar-scheduler-sm': {
              maxWidth: 410
          }
      });
  };
  HonCalendarScheduler.prototype = Object.create(CompositeEditor.prototype);
  HonCalendarScheduler.prototype.constructor = HonCalendarScheduler;

  HonCalendarScheduler.prototype.makeBuilder = function () {
    var _this = this;

    var builder = new CompositeBuilder();
    builder.getKeys = function () {
      return ['calendarScheduler', 'activeDates'];
    };
    builder.getDomFor = function (key) {
      return _this.jq().children('.' + key);
    };
    builder.getConfigFor = function (key) {
      switch (key) {
        case 'calendarScheduler':
          return {
            type: CalendarEventsEditor,
            properties: { includeReference: _this.$includeReference(), sourceOrd: _this.$sourceOrd }
          };
        case 'activeDates':
          return {
            properties: { refBase: _this.$refBase, sourceOrd: _this.$sourceOrd },
            type: ActiveDatesCalendar
          };
      }
    };
    builder.getValueFor = function (key) {
      switch (key) {
        case 'calendarScheduler':
        case 'activeDates':
          return _this.$snapshot;
      }
    };
    return builder;
  };

  /**
   * Return false not include ScheduleReference as an option when
   * adding a new Event. This prevents the CalendarScheduler from creating
   * a reference to itself.
   * @returns {boolean}
   * @private
   */
  HonCalendarScheduler.prototype.$includeReference = function () {
    return false;
  };

  HonCalendarScheduler.prototype.doInitialize = function (dom) {
    var _this2 = this;

    dom.addClass('CalendarScheduler').html(tpl(this));

    dom.on(MODIFY_EVENT, '.calendarScheduler', function () {
      _this2.setModified(true);
      return false;
    });
  };

  /**
   * @param {baja.Component} value a `schedule:CalendarSchedule`
   * @returns {Promise}
   */
  HonCalendarScheduler.prototype.doLoad = function (value) {
    var _this3 = this;
    _calendar = value;
    var args = arguments;
    var navOrd = value.getNavOrd();

    return ssc.loadScheduleInfo(navOrd).then(function (_ref) {
      var snapshot = _ref.snapshot,
          refBase = _ref.refBase,
          options = _ref.options;

      _this3.$refBase = refBase;
      _this3.$snapshot = snapshot;
      _this3.$sourceOrd = navOrd;
      _this3.$options = options;
      return CompositeEditor.prototype.doLoad.apply(_this3, args);
    }).then(function () {
      _this3.$getCalendarEventsEditor().on('selectedEventChanged', function (event) {
        var sched = event || _this3.$getCalendarEventsEditor().value();
        _this3.$getActiveDatesCalendar().load(sched).catch(Log.error);
      });
       _calendar.serverSideCall({
                  typeSpec: "honBACnetUtilities:HonScheduleServerSideCallHandler",
                  methodName: "readCalendar",
                  value: "", // This is the argument for the Server Side Call
            });

      if (_this3.$options.readonly) {
        return _this3.setReadonly(true);
      }
    });
  };

  HonCalendarScheduler.prototype.doDestroy = function () {
    this.jq().removeClass('CalendarScheduler');
    return CompositeEditor.prototype.doDestroy.apply(this, arguments);
  };

  HonCalendarScheduler.prototype.$getActiveDatesCalendar = function () {
    return this.getBuilder().getEditorFor('activeDates');
  };

  HonCalendarScheduler.prototype.$getCalendarEventsEditor = function () {
    return this.getBuilder().getEditorFor('calendarScheduler');
  };

  HonCalendarScheduler.prototype.$doSaveCalendarEvents = function () {
    return this.$getCalendarEventsEditor().save();
  };

  HonCalendarScheduler.prototype.doSave = function () {
    var _this4 = this;

    return this.$doSaveCalendarEvents().then(function () {
      return ssc.save(_this4.value().getNavOrd(), _this4.$snapshot);
    })
    .then(function(){
       _calendar.serverSideCall({
                      typeSpec: "honBACnetUtilities:HonScheduleServerSideCallHandler",
                      methodName: "saveCalendar",
                      value: "", // This is the argument for the Server Side Call
                });
      });
  };

  return HonCalendarScheduler;
});
