/*
 *  Copyright (c) 2020 Honeywell International, Inc. All rights reserved.
 *
 *  This file contains trade secrets of Honeywell International, Inc.  No part
 *  may be reproduced or transmitted in any form by any means or for any
 *  purpose without the express written permission of Honeywell.
 */
/* globals console */ // jshint ignore:line
                      // prevents grunt from complaining,
                      // not needed once console (ie console.log) is used
/**
 * A module defining `HonDataPointView`.
 *
 * @module nmodule/honBACnetUtilities/rc/HonDataPointView
 */
 define(['baja!',
 'bajaux/Widget',
 'bajaux/util/SaveCommand',
 'dialogs',
 'bajaux/mixin/subscriberMixIn',
 'jquery',
 "bajaScript/baja/ord/SlotPath",
 'Promise',
 'baja!bacnet:BacnetObjectType',
 'baja!bacnet:BacnetListOf',
 'baja!bacnet:BacnetVirtualObject',
 'nmodule/honBACnetUtilities/rc/HonBacnetUtils',
 'nmodule/honBACnetUtilities/rc/lib/react.production.min',
 'nmodule/honBACnetUtilities/rc/lib/react-dom.production.min',
 'nmodule/honBACnetUtilities/rc/es6/components/honbacnetprofile',
 'hbs!nmodule/honBACnetUtilities/rc/templates/home',
 'lex!honBACnetUtilities',
 'css!nmodule/honBACnetUtilities/rc/css/honBACnetUtilities',
 'css!nmodule/honBACnetUtilities/rc/css/bootstrap.min'], function (baja,
                                                                   Widget,
                                                                   SaveCommand,
                                                                   dialogs,
                                                                   subscriberMixIn,
                                                                   $,
                                                                   SlotPath,
                                                                   Promise,
                                                                   BacnetObjectType,
                                                                   BacnetListOf,
                                                                   BacnetVirtualObject,
                                                                   HonBacnetUtils,
                                                                   React,
                                                                   ReactDOM,
                                                                   honbacnetprofile,
                                                                   homeTemplate,
                                                                   lexicons) {

 'use strict';

 var lex = lexicons[0];
 /**
  * Description of your widget.
  *
  * @class
  * @extends module:bajaux/Widget
  * @alias module:nmodule/honBACnetUtilities/rc/HonDataPointView
  */
 var HonDataPointView = function () {
     var that = this;

     // This calls the super constructor.
     Widget.apply(that, arguments);

     // Applying a Subscriber MixIn adds subscription capabilities to a
     // Widget. Once added, the value passed into 'doLoad' is fully subscribed
     // and 'live'.
     subscriberMixIn(that);
     that.getCommandGroup().add(new SaveCommand());
 };

 //extend and set up prototype chain
 HonDataPointView.prototype = Object.create(Widget.prototype);
 HonDataPointView.prototype.constructor = HonDataPointView;
 var domObjects = [], widgetCounter = 0, pointDetail = [];
 var modifiedWidgets = [];
 //var presentValuePoint = null;
 var isSaveObj = false;
////////////////////////////////////////////////////////////////
// Widget
////////////////////////////////////////////////////////////////

 /**
  * Describe how your widget does its initial setup of the DOM.
  *
  * @param {jQuery} jq The DOM element into which to load this Widget
  */
 HonDataPointView.prototype.doInitialize = function (dom) {
     domObjects.push(dom);
 };

 /**
  * Describe how your widget loads in a value.
  *
  * @param {baja.Component} in this case the enum component loaded into the widget.
  */
 HonDataPointView.prototype.doLoad = function (dataPoint) {
     var that = this;
     var pointDetailObj = {name:"", bacnetVirtualObject: null ,pvPoint: null,slotPath:'', handleOrd: ''};
     modifiedWidgets = [];
     var widgetDom = this.$jq; //domObjects[widgetCounter++];
     if(widgetDom == null){
         widgetDom = domObjects[0];
     }
     widgetDom.html(homeTemplate({pointName: dataPoint.getName()}));
     var reactDoms = document.getElementsByName('honbacnetUtilityHome');
     var reactDom = null;
     for (var x = 0; x < reactDoms.length; x++) {
         if (!reactDoms[x].classList.contains("reactUsed") && reactDoms[x].outerHTML.indexOf(dataPoint.getName()) > -1) {
             reactDom = reactDoms[x];
             reactDom.classList.add("reactUsed");
             reactDom.setAttribute("id", SlotPath.unescape(dataPoint.getName()).replace(" ", ""));
             break;
         }
     }
     pointDetailObj.name = dataPoint.getDisplayName();
     pointDetailObj.pvPoint = dataPoint;
     pointDetailObj.slotPath = dataPoint.getSlotPath().getBody();
     pointDetailObj.handleOrd = dataPoint.getSlotPath().getBody();
     pointDetail.push(pointDetailObj);
     var setBacnetVirtualObject = function (obj){
         //bacnetVirtualObject = obj;
         var nameOfPoint = obj.pointName;
         var handleOfPoint = obj.handleOrd;
         var temp = pointDetail.find(function (element){
             return element.handleOrd === handleOfPoint;
         });
         if(temp !==null && temp!==undefined ){
             temp.bacnetVirtualObject = obj.bacObject;
         }
     }
     var componentData = function (data) {
         var obj = null;
         for (var i = 0; i < modifiedWidgets.length; i++) {
             if (modifiedWidgets[i].propertyId === data.propertyId && modifiedWidgets[i].slotPath === data.slotPath) {
                 obj = modifiedWidgets[i];
                 modifiedWidgets[i] = data;
                 break;
             }
         }
         if (obj == null) {
             modifiedWidgets.push(data);
         }
         that.setModified(true);
         if(data.propertyId === HonBacnetUtils.CHANGE_OF_STATE_COUNT ||
             data.propertyId === HonBacnetUtils.ELAPSED_ACTIVE_TIME
             ){
             that.doSave();
             that.setModified(false);
         }
     }

     var props = {
         point: {name: dataPoint.getName(), ord: dataPoint.getNavOrd(),displayName: dataPoint.getDisplayName()},
         dataPoint: dataPoint,
         notifyData: componentData,
         updateBacnetObject: setBacnetVirtualObject,
         widgetThat: that
     };
     ReactDOM.render(React.createElement(honbacnetprofile, props), reactDom);
 };
 var getModifiedProperty = function (propertyId, strSlotPath) {
     var modifiedFlag = null;
     for (var i = 0; i < modifiedWidgets.length; i++) {
         if (modifiedWidgets[i].propertyId == propertyId && SlotPath.unescape(modifiedWidgets[i].slotPath) == SlotPath.unescape(strSlotPath)) {
             modifiedFlag = modifiedWidgets[i];
             break;
         }
     }
     return modifiedFlag;
 };
 HonDataPointView.prototype.doRead = function () {
 };

 HonDataPointView.prototype.getSaveObj = function () {
     return isSaveObj;
 };

 HonDataPointView.prototype.setSaveObj = function (val) {
     isSaveObj = val;
 };


 /**
  * Save the user-entered changes to the loaded Ramp. Note that the parameter
  * to this function is the same as that resolved by doRead().
  *
  * @returns {Promise}
  */
 HonDataPointView.prototype.doSave = function (readValue) {
     //var jq = this.jq(),
     //this.jq().prevObject[0].innerHTML.indexOf("E1_CS42_E2_AI1")
     var currentPointSlotPath = "";
     var reloadPage = false;
     var presentValuePoint = null;
     var bacnetVirtualObject = baja.$("bacnet:BacnetVirtualObject");
     var htmlObject = this.jq().prevObject != undefined?this.jq().prevObject[0] : this.jq()[0];
    if(htmlObject != undefined) {
        for (var y = 0; y < pointDetail.length; y++) {
            if (htmlObject.innerHTML.indexOf(pointDetail[y].slotPath) >= 0) {
                presentValuePoint = pointDetail[y].pvPoint;
                bacnetVirtualObject = pointDetail[y].bacnetVirtualObject;
                currentPointSlotPath = pointDetail[y].slotPath;
                break;
            }
        }
    }else{
        presentValuePoint = pointDetail[0].pvPoint;
        bacnetVirtualObject = pointDetail[0].bacnetVirtualObject;
        currentPointSlotPath = pointDetail[0].slotPath;
    }
     isSaveObj = true;
       var  batch = new baja.comm.Batch(),
         promises = [];
    var  presentValueProp = getModifiedProperty(HonBacnetUtils.PRESENT_VALUE, currentPointSlotPath);
      if (presentValueProp != null) {

          var autoMode = presentValueProp.mode,
              msg = "";

          if (autoMode) {
              msg = "doSave: setNiagaraPointToAuto: ";
              //baja.outln(msg);
              presentValuePoint.serverSideCall({
                  typeSpec: "honBACnetUtilities:HonServerSideCallHandler",
                  methodName: "setPointToAuto",
                  value: "", // This is the argument for the Server Side Call
              }).then(function (response) {
              });
          } else {
              msg = "doSave: setNiagaraPointToManual: ";
              //baja.outln(msg);

              var presentValue = null;

              if (!presentValueProp.isMultiState) {
                  presentValue = baja.$('baja:Float').make(parseFloat( presentValueProp.propertyValue.value));
              } else if ((presentValueProp.type === "binaryInput") || 
              (presentValueProp.type === "binaryValue") ||
              (presentValueProp.type === 'binaryPv') || 
              (presentValueProp.type === 'binaryOutput')) {
                  var binaryPv = baja.$('bacnet:BacnetBinaryPv').make(parseInt(presentValueProp.propertyValue.value, 10));
                  presentValue = baja.$('baja:Boolean').make(binaryPv.isActive());
              } else {
                  presentValue = baja.$('bacnet:BacnetUnsigned').make(parseInt(presentValueProp.propertyValue.value, 10));
              }
              presentValuePoint.serverSideCall({
                  typeSpec: "honBACnetUtilities:HonServerSideCallHandler",
                  methodName: "setPointToManual",
                  value: presentValue, // This is the argument for the Server Side Call
              }).then(function (response) {
                    reloadPage = true;
              });
          }
      }

     if (bacnetVirtualObject) {
          bacnetVirtualObject.getSlots().properties().isComponent().each(function (slot) {
              if (slot.getType().is("bacnet:BacnetVirtualProperty")) {
                  var prop = bacnetVirtualObject.get(slot),
                      id = prop.getPropertyId();
                  var propModifiedValues = getModifiedProperty(id,currentPointSlotPath);
                  if (propModifiedValues !== null) {
                      prop.loadSlots();
                      var name = "" + prop.getName(),
                          widgetId = name + "Id";

                      //This is the widget to be updated
                      var msg = "doSave: ";
                      msg += "property: " + name;
                      msg += "; widgetId: " + widgetId;
                      //baja.outln(msg);
                      //var widget = jq.find('#' + widgetId);
                      var bitStr, listOf, i, slotName ,idx;
                      switch (id) {
                          case HonBacnetUtils.PRESENT_VALUE:
                              break;
                          case HonBacnetUtils.RELINQUISH_DEFAULT:
                             // promises.push(prop.set({
                             //     slot: 'value',
                             //     value: baja.$('baja:Float').make(parseFloat(propModifiedValues.propertyValue.value)),
                             //     batch: batch
                             // }));
                                if(autoMode) {
                                    if (propModifiedValues.isNumeric) {
                                        promises.push(prop.set({
                                            slot: 'value',
                                            value: baja.$('baja:Float').make(parseFloat(propModifiedValues.propertyValue.value)),
                                            batch: batch
                                        }));

                                    } else if (propModifiedValues.isBinary) {
                                        promises.push(prop.set({
                                            slot: 'value',
                                            value: baja.$('bacnet:BacnetBinaryPv').make(parseFloat(propModifiedValues.propertyValue.value, 10)),
                                            batch: batch
                                        }));

                                    } else {
                                        promises.push(prop.set({
                                            slot: 'value',
                                            value: baja.$('bacnet:BacnetUnsigned').make(parseInt(propModifiedValues.propertyValue.value, 10)),
                                            batch: batch
                                        }));
                                    }
                                }else{
                                    var val = '';
                                    if(propModifiedValues.isNumeric) {
                                        val = parseFloat(propModifiedValues.propertyValue.value);
                                    } else if(propModifiedValues.isBinary){
                                        val= baja.$('bacnet:BacnetBinaryPv').make(parseFloat(propModifiedValues.propertyValue.value, 10));
                                    }else{
                                        val = baja.$('bacnet:BacnetUnsigned').make(parseInt(propModifiedValues.propertyValue.value, 10));
                                    }
                                    if(val !='') {
                                        presentValuePoint.serverSideCall({
                                            typeSpec: "honBACnetUtilities:HonServerSideCallHandler",
                                            methodName: "setPriorityManual",
                                            value: val
                                        }).then(function (response) {
                                            if(!reloadPage){
                                                reloadPage  = true;
                                            }
                                        });
                                    }
                                }
                             break;
                            
                          case HonBacnetUtils.ALARM_VALUE:
                              if (propModifiedValues.isNumeric) {
                                  promises.push(prop.set({
                                      slot: 'value',
                                      value: baja.$('baja:Float').make(parseFloat(propModifiedValues.propertyValue.value)),
                                      batch: batch
                                  }));
                              } else if (propModifiedValues.isBinary) {
                                  promises.push(prop.set({
                                      slot: 'value',
                                      value: baja.$('bacnet:BacnetBinaryPv').make(parseInt(propModifiedValues.propertyValue.value, 10)),
                                      batch: batch
                                  }));
                              } else {
                                  promises.push(prop.set({
                                      slot: 'value',
                                      value: baja.$('bacnet:BacnetUnsigned').make(parseInt(propModifiedValues.propertyValue.value, 10)),
                                      batch: batch
                                  }));
                              }
                              break;
                          case HonBacnetUtils.ERROR_LIMIT:
                          case HonBacnetUtils.DEADBAND:
                          case HonBacnetUtils.COV_INCREMENT:
                          case HonBacnetUtils.HIGH_LIMIT:
                          case HonBacnetUtils.LOW_LIMIT:
                          case HonBacnetUtils.RESOLUTION:
                          case HonBacnetUtils.SAFETY_VALUE:
                          //case HonBacnetUtils.SENSOR_OFFSET:
                          case HonBacnetUtils.PROPORTIONAL_VARANT:
                          case HonBacnetUtils.INTEGRAL_VARANT:
                          case HonBacnetUtils.DERIVATIVE_VARANT:
                          case HonBacnetUtils.BIAS:
                          case HonBacnetUtils.MAXIMUM_OUTPUT:
                          case HonBacnetUtils.MINIMUM_OUTPUT:
                              promises.push(prop.set({
                                  slot: 'value',
                                  value: baja.$('baja:Float').make(parseFloat(propModifiedValues.propertyValue.value)),
                                  batch: batch
                              }));
                              break;

                          case HonBacnetUtils.TIME_DELAY:
                          case HonBacnetUtils.TIME_TO_OPEN:
                          case HonBacnetUtils.TIME_TO_CLOSE:
                          case HonBacnetUtils.TIME_DELAY_NORMAL:
                          case HonBacnetUtils.NOTIFICATION_CLASS:
                              promises.push(prop.set({
                                  slot: 'value',
                                  value: baja.$('bacnet:BacnetUnsigned').make(parseInt(propModifiedValues.propertyValue.value, 10)),
                                  batch: batch,
                              }));
                              break;
                          case HonBacnetUtils.SAFETY_POSITION:
                          case HonBacnetUtils.DIRECT_REVERSE:
                              promises.push(prop.set({
                                  slot: 'value',
                                  value: baja.$('bacnet:BacnetUnsigned').make(parseInt(propModifiedValues.propertyValue.value, 10)),
                                  batch: batch,
                              }));
                              break;

                          case HonBacnetUtils.CHANGE_OF_STATE_COUNT:
                          case HonBacnetUtils.ELAPSED_ACTIVE_TIME:
                              promises.push(prop.set({
                                  slot: 'value',
                                  value: baja.$('bacnet:BacnetUnsigned').make("0"),
                                  batch: batch,
                              }));
                              break;

                          case HonBacnetUtils.OUT_OF_SERVICE:
                              var isOutOfService = false;
                              if (parseInt(propModifiedValues.propertyValue.value) === 1) {
                                  isOutOfService = true;
                              }
                              promises.push(prop.set({
                                  slot: 'value',
                                  value: baja.$('baja:Boolean').make(isOutOfService),
                                  batch: batch
                              }));
                              break;

                          case HonBacnetUtils.LIMIT_ENABLE:
                              bitStr = "";
                              bitStr = (propModifiedValues.propertyValue.lowLimitEnable ? "1" : "0");
                              bitStr += (propModifiedValues.propertyValue.highLimitEnable ? "1" : "0");
                              promises.push(prop.set({
                                  slot: 'value',
                                  value: baja.$('bacnet:BacnetBitString').make(bitStr),
                                  batch: batch,
                              }));
                              break;


                          case HonBacnetUtils.EVENT_ENABLE:
                              bitStr = "";
                              bitStr = propModifiedValues.propertyValue.toOffNormal ? "1" : "0";
                              bitStr += propModifiedValues.propertyValue.toFault ? "1" : "0";
                              bitStr += propModifiedValues.propertyValue.toNormal ? "1" : "0";
                              promises.push(prop.set({
                                  slot: 'value',
                                  value: baja.$('bacnet:BacnetBitString').make(bitStr),
                                  batch: batch,
                              }));
                              break;

                         case HonBacnetUtils.FAULT_VALUES:
                         case HonBacnetUtils.ALARM_VALUES:
                             listOf = baja.$("bacnet:BacnetListOf");
                             listOf.setListTypeSpec(baja.$('baja:TypeSpec').make("bacnet:BacnetUnsigned"));
                             for (i = 0; i < propModifiedValues.propertyValue.length; i++) {
                                 listOf.add({
                                      slot: "BacnetUnsigned" + index > 0 && index,
                                      value: baja.$('bacnet:BacnetUnsigned').make(parseInt("" + propModifiedValues.propertyValue[i] ? 1 : 0, 10)),
                                 });
                             }
                             promises.push(prop.set({
                                 slot: 'value',
                                 value: listOf,
                                 batch: batch,
                             }));
                             break;
                          case HonBacnetUtils.SENSOR_OFFSET:
                              presentValuePoint.serverSideCall({
                                  typeSpec: "honBACnetUtilities:HonServerSideCallHandler",
                                  methodName: "setPointOffset",
                                  value: parseFloat(propModifiedValues.propertyValue.value)
                              }).then(function (response) {
                              });
                              break;
                           //  listOf = baja.$("bacnet:BacnetListOf");
                             //  listOf.setListTypeSpec(baja.$('baja:TypeSpec').make("bacnet:BacnetUnsigned"));

                             //  for (i = 1; i <= $('#presentValueId > option').length; i++) {
                             //      if (jq.find('#alarmValues_' + i.toString()).prop('checked')) {
                             //          slotName = "BacnetUnsigned";
                             //          slotName += (i === 1) ? "" : "" + (i - 1);

                             //          listOf.add({
                             //              slot: slotName,
                             //              value: baja.$('bacnet:BacnetUnsigned').make(parseInt("" + i, 10)),
                             //          });
                             //      }
                             //  }

                             //  promises.push(prop.set({
                             //      slot: 'value',
                             //      value: listOf,
                             //      batch: batch,
                             //  }));
                            //  break;

                         //  case HonBacnetUtils.FAULT_VALUES:
                         //      listOf = baja.$("bacnet:BacnetListOf");
                         //      listOf.setListTypeSpec(baja.$('baja:TypeSpec').make("bacnet:BacnetUnsigned"));

                         //      for (i = 1; i <= $('#presentValueId > option').length; i++) {
                         //          if (jq.find('#faultValues_' + i.toString()).prop('checked')) {
                         //              slotName = "BacnetUnsigned";
                         //              slotName += (i === 1) ? "" : "" + (i - 1);

                         //              listOf.add({
                         //                  slot: slotName,
                         //                  value: baja.$('bacnet:BacnetUnsigned').make(parseInt("" + i, 10)),
                         //              });
                         //          }
                         //      }

                         //      promises.push(prop.set({
                         //          slot: 'value',
                         //          value: listOf,
                         //          batch: batch,
                         //      }));
                         //      break;

                          default:
                              break;
                      }
                  }
              }
          });
      }

      // Commit the changes in one network call.
      batch.commit();
     // modifiedWidgets = [];
      // Return a Promise so that the framework knows when the save has completed.
     return Promise.all(promises);
 };

 HonDataPointView.prototype.doDestroy = function () {

     //unsubscribe from device and from virtuals
     //if (bacnetDevice) {
     //}

     /*if (presentValuePoint) {
         this.getSubscriber().detach("changed");
     }*/

 };
 return HonDataPointView;
});
