/*
 *
 *  * Copyright (c) 2019. Honeywell International, Inc. All Rights Reserved.
 *    created by H148225 on 7/19/2019
 *
 */

define(['baja!',
        'jquery',
        'underscore',
        'Promise',
        'dialogs',
        'nmodule/easyDatabaseManager/rc/js/defaultView',
        'nmodule/easyDatabaseManager/rc/fe/baja/SecuredDeleteCommand',
        'lex!easyDatabaseManager',
        'hbs!nmodule/easyDatabaseManager/rc/hbs/deletePointsViewTemplate',
        'hbs!nmodule/easyDatabaseManager/rc/hbs/deletePointsSummary',
        'css!nmodule/easyDatabaseManager/rc/css/datatable',
        'css!nmodule/easyDatabaseManager/rc/css/styles'],
    function (baja,
              $,
              _,
              promise,
              dialogs,
              defaultView,
              SecuredDeleteCommand,
              lexicons,
              template,
              dpSummary) {

        'use strict';
        var lex = lexicons[0];
        var dom;
        // deletePointsView step
        // arguments: dom, vname, id, message, summaryMessage, backBttnMsg, nextBttnMsg, cancelBttnMsg
        var deletePointsView = function () {
            defaultView.apply(this, arguments);

            //update which buttons will be available
            this.buttonstatus = {
                disabledBack: false,
                disabledNext: false
            };
        };
        deletePointsView.prototype = Object.create(defaultView.prototype,
            {'constructor': deletePointsView}
        );

        //these variables will be updated in loadView with values passed from PointDatabaseManagerWidget
        deletePointsView.component = null;
        deletePointsView.service = null;
        deletePointsView.subscriber = null;
        deletePointsView.isStationBackupChecked = true;
        deletePointsView.isSelectedDeletedRecordsLogChecked = true;
        //deletePointsView.isSecurePointAuthenticated = false;

        // call this function anytime you want to update nav action button status to enable/disable them
        deletePointsView.updateActionButtonsStatus = null;

        var _addEventListeners = function (element) {
            //select or unselect individual rows
            $(element).find("div.ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default").on('click',
                function (event) {
                    var $target = $(event.target),
                        checkboxState,
                        targetID;
                    if ($target.hasClass("ui-chkbox-icon")) {
                        $target.parent().toggleClass("ui-state-active");
                        $target.toggleClass("pi pi-check");
                        checkboxState = false;
                        targetID = $target.parent().attr('id');
                    } else {
                        $target.toggleClass("ui-state-active");
                        $target.find(".ui-chkbox-icon.ui-clickable").toggleClass("pi pi-check");
                        checkboxState = true;
                        targetID = $target.attr('id');
                    }

                    if ("stationBackup" === targetID) {
                        deletePointsView.isStationBackupChecked = checkboxState;
                    }
                    if ("selectedDeletedRecordsLog" === targetID) {
                        deletePointsView.isSelectedDeletedRecordsLogChecked = checkboxState;
                    }
                });
            /**
             * show/hide points details on click of + event
             */
            $(element).find('#showpointdetails').on('click',
                function (event) {
                    if (this.src.includes("expand.png")) {
                        this.setAttribute("src", "/module/easyDatabaseManager/rc/images/collapse.png");
                    } else {
                        this.setAttribute("src", "/module/easyDatabaseManager/rc/images/expand.png");
                    }
                    var pointDetails = $(element).find('#pointDetails');
                    pointDetails.toggleClass("PDM-hide-element");
                });
        };

        /**
         * Load view of current wizard step - deletePointsView
         * @param element: dom element the view will be appended to.
         * @param params: javascript object with widget parameters required to load the view within the widget
         */
        deletePointsView.prototype.loadView = function (element, params) {
            dom = element;
            deletePointsView.component = params.component;
            deletePointsView.service = params.service;
            deletePointsView.subscriber = params.subscriber;
            deletePointsView.updateActionButtonsStatus = params.updateActionButtonsStatus;
            deletePointsView.isStationBackupChecked = true;
            deletePointsView.isSelectedDeletedRecordsLogChecked = true;
            //deletePointsView.isSecurePointAuthenticated = false;
            this.$pointDeletionSummary();

            $(element).append(template({
                    vid: this.params.vid,
                    summaryMessage: this.params.summaryMessage,
                    stepCount: this.params.stepCount,
                    stepDescription: this.params.stepDescription,
                    stationBackupConfirmationMessage: lex.get("pdm.PointDatabaseManager.stationBackupConfirmationMsg"),
                    selectedDeletedRecordsLogConfirmationMessage: lex.get("pdm.PointDatabaseManager.selectedDeletedRecordsLogConfirmationMsg")
                }
            ));
            $(element).find(".PDM-contents-step-summary").css("border-top-color", params.colorCode);
            var checkboxContainer = dom.find('#PDM-step5-stationbackup-logs-checkbox-container');
            _addEventListeners(checkboxContainer);
        };

        deletePointsView.prototype.showLoadedView = function () {
            //Update variables on back
            deletePointsView.updateActionButtonsStatus(
                $(dom).parent(),
                {
                    disabledBack: false,
                    disabledNext: false
                }
            );
        };

        deletePointsView.prototype.$pointDeletionSummary = function () {
            var that = this;
            baja.rpc({
                typeSpec: 'easyDatabaseManager:PointDataManagerRpc',
                method: 'getDeletePointsSummary'
            })
                .then(function (list) {
                    that.$updateSummaryTable(list);
                })

        };
        deletePointsView.prototype.$updateSummaryTable = function (deletePointsList) {
            var networkList = JSON.parse(deletePointsList);
            var that = this;
            var deletePointsSummaryStage = that.dom.find('.PDM-contents-deletion-summary-container');
            var count = 0;
            var rowElem = $('<div class="PDM-contents-deletion-summary-container-row"></div>');
            _.each(networkList, function (network) {
                deletePointsSummaryStage.append(rowElem);
                var elem = $(dpSummary({
                    networkName: network.networkName,
                    totalPointsToDelete: network.totalPointsToDelete,
                    totalTrendPointsToDelete: network.totalTrendPointsToDelete,
                    totalNiagaraPointsToDelete: network.totalNiagaraPointsToDelete,
                    totalUnusedAndUnwantedTrendPoints: network.totalUnusedAndUnwantedTrendPoints,
                    totalUnusedAndUnwantedNiagaraPoints: network.totalUnusedAndUnwantedNiagaraPoints,
                    totalUnusedAndUnwantedPoints: network.totalUsedAndUnusedPoints,
                    totalHistories: network.totalHistories,
                    totalHistoriesToDelete: network.totalHistoriesToDelete,
                    totalSchedules: network.totalSchedules,
                    totalSchedulesToDelete: network.totalSchedulesToDelete,
                    deletingPoints: lex.get("pdm.PointDatabaseManager.deletePointsView.deletingPoints"),
                    deleteText: lex.get("pdm.PointDatabaseManager.deletePointsView.deleteText"),
                    histories: lex.get("pdm.PointDatabaseManager.deletePointsView.histories"),
                    schedules: lex.get("pdm.PointDatabaseManager.deletePointsView.schedules"),
                    of: lex.get("pdm.PointDatabaseManager.deletePointsView.of"),
                    trendPoints: lex.get("pdm.PointDatabaseManager.deletePointsView.trendPoints"),
                    niagaraPoints: lex.get("pdm.PointDatabaseManager.deletePointsView.niagaraPoints"),
                    selectedSecurePoint: lex.get("pdm.PointDatabaseManager.deletePointsView.selectedSecurePoint"),
                    securePointPath: network.securePointSelected
                }));

                /**
                 * addeventlisteners onclick of + icon
                 */
                rowElem.append(elem);

                var elemShowPointDetails = elem.find('#showpointdetails');
                var pointDetails = elem.find('#pointDetails');
                pointDetails.addClass("PDM-hide-element");
                if (network.networkType !== undefined) {
                    if (network.networkType === "TrendN4:TrendNetwork") {
                        elemShowPointDetails.removeClass("PDM-hide-element");
                        _addEventListeners(elem);
                    } else {
                        elemShowPointDetails.addClass("PDM-hide-element");
                    }
                }
                if (network.securePointSelected !== undefined) {
                    if (network.securePointSelected == "null"){
                        elem.find('.selectedSecurePoint').hide()
                    }
                }

            });

        };

        /**
         * validate user input before proceeding to next screen.
         * This is called before view.prototype.next is called.
         *
         * @returns {boolean}: return true/false based on validity of user input
         */
        deletePointsView.prototype.validate = function () {
            baja.outln("inside: deletePointsView.prototype.validate ...");
            return new promise(function (resolve) {
                //do any validations needed here.
                //resolve once all the input is validated and good to move to next screen
                resolve(true);
            });
        };

        var startDeletingPoints = function (resolve) {
            baja.rpc({
                typeSpec: 'easyDatabaseManager:PointDataManagerRpc',
                method: 'deleteSelectedItems',
                args: [{
                    isMaintainLog: deletePointsView.isSelectedDeletedRecordsLogChecked,
                    isSecurePointAuthenticated: false,
                    pointTobeDeleted: null
                }]
            }).then(function (ord) {
                var job = baja.Ord.make("station:|" + ord).get({lease: true});
                return job;
            }).then(completeJob)
                .then(function () {
                        resolve(true);
                    },
                    function () {
                        //notify user.
                        notifyUser(lex.get("pdm.PointDatabaseManager.deletePointsView.delete.error"));
                    }
                );
        };

        var startDeletingSecurePoints = function (param, resolve) {
            baja.rpc({
                typeSpec: 'easyDatabaseManager:PointDataManagerRpc',
                method: 'deleteSelectedItems',
                args: [{
                    isMaintainLog: deletePointsView.isSelectedDeletedRecordsLogChecked,
                    isSecurePointAuthenticated: true,
                    pointTobeDeleted: param
                }]
            }).then(function (ord) {
                var job = baja.Ord.make("station:|" + ord).get({lease: true});
                return job;
            }).then(completeJob)
                .then(function () {
                        resolve(true);
                    },
                    function () {
                        //notify user.
                        notifyUser(lex.get("pdm.PointDatabaseManager.deletePointsView.delete.error"));
                    }
                );
        };

        var doElectronicSignature = function (pointOrdStr, resolve) {
            baja.Ord.make(pointOrdStr).get({lease: true}).then(function (point) {
                var strArray = pointOrdStr.split("/")
                var pointName = strArray[strArray.length - 1];
                var securedDeleteCommand = new SecuredDeleteCommand();
                var securedPointDeletePromise = new promise(function (resolve) {
                    securedDeleteCommand.performCommand(point.getParent(), [point.getParent().getSlot(pointName)], pointOrdStr, resolve)
                })
                securedPointDeletePromise.then(function (successMessage) {
                    if (successMessage == "ok") {
                        startDeletingSecurePoints(pointOrdStr, resolve);
                        startDeletingPoints(resolve);
                    } else {
                        deletePointsView.updateActionButtonsStatus(
                            $(dom).parent(),
                            {
                                disabledBack: false,
                                disabledNext: false
                            }
                        );
                        resolve(false);
                    }
                });
            });
        }

        var checkDeletingPoints = function (resolve) {
            var pointOrdStr;
            var checkDeletingPointsPromise = new promise(function (resolve) {
                baja.Ord.make("file:^edm/edm.xml").get({lease: true})
                    .then(function (response) {
                        var i, point, pointList;
                        $.get(response.getReadUri()).then(function (points) {
                            pointList = $(points).find('Item');
                            if (pointList !== null && pointList.length > 0) {
                                for (i = 0; i < pointList.length; i++) {
                                    point = pointList[i];
                                    if ($(point).attr("secure") && $(point).attr("secure") === "1" && $(point).attr("delete") === "1") {
                                        pointOrdStr = "station:|" + $(point).attr("path");
                                        resolve("SecurePoint");
                                        break;
                                    }
                                }
                            }
                            resolve("NonSecurePoint");
                        });
                    });
                })
                checkDeletingPointsPromise.then(function (successMessage) {
                if (successMessage == "SecurePoint") {
                    doElectronicSignature(pointOrdStr, resolve);
                }
                if (successMessage == "NonSecurePoint") {
                    startDeletingPoints(resolve);
                }
            });
        }

        var startProcess = function (resolve) {
            $(Element).find("#stationBackup").unbind();
            $(Element).find("#selectedDeletedRecordsLog").unbind();
            if (deletePointsView.isStationBackupChecked) {
                baja.rpc({
                    typeSpec: 'easyDatabaseManager:PointDataManagerRpc',
                    method: 'stationBackup',
                    args: [{}]
                }).then(function (ord) {
                    var job = baja.Ord.make("station:|" + ord).get({lease: true});
                    return job;
                }).then(completeJob)
                    .then(function () {
                            checkDeletingPoints(resolve);
                        },
                        function () {
                            //notify user.
                            notifyUser(lex.get('pdm.PointDatabaseManager.deletePointsView.backup.error'));

                            deletePointsView.updateActionButtonsStatus(
                                $(dom).parent(),
                                {
                                    disabledBack: false,
                                    disabledNext: false
                                }
                            );

                        }
                    );
            } else {
                checkDeletingPoints(resolve);
            }
        }

        /**
         * clean up and save before going to next view
         *
         */
        deletePointsView.prototype.next = function () {
            var confirmationDialog;
            baja.outln("inside: deletePointsView.prototype.next ...");

            deletePointsView.updateActionButtonsStatus(
                $(dom).parent(),
                {
                    disabledBack: true,
                    disabledNext: true
                }
            );

            return new promise(function (resolve) {
                // do any backend communication needed here before moving to next screen.
                //resolve once all the input is validated and good to move to next screen
                confirmationDialog = dialogs.make({
                        title: lex.get("pdm.PointDatabaseManager.stepFive.confirm.dialog.title"),
                        content: lex.get("pdm.PointDatabaseManager.stepFive.confirm.dialog.content"),
                        layout: function (dlg) {
                            var $dialog = dlg.jq();
                            $dialog.find("div.js-dialog.ux-root").css({"width": "30%"});
                            $dialog.find("div.js-dialog-content").css({
                                "padding-top": "15%",
                                "padding-bottom": "10%",
                                "padding-left": "10px",
                                "font-weight": "800"
                            });
                            $dialog.find("button.ux-btn.js-dialog-button.js-dialog-button-cancel").css({"color": "grey"});
                            $dialog.find("ul.js-dialog-button-content").css({"padding-bottom": "35px"});
                        },
                        buttons: [{
                            name: "cancel",
                            displayName: lex.get("pdm.PointDatabaseManager.button.cancel"),
                            handler: function (dlg) {
                                deletePointsView.updateActionButtonsStatus(
                                    $(dom).parent(),
                                    {
                                        disabledBack: false,
                                        disabledNext: false
                                    }
                                );
                                resolve(false);
                            }
                        },
                            {
                                name: "remove",
                                displayName: lex.get("pdm.PointDatabaseManager.button.remove"),
                                handler: function (dlg) {
                                    startProcess(resolve);
                                    dlg.close();
                                }
                            }]
                    }
                );
                confirmationDialog.show();
            });
        };

        /**
         * clean up and save before going to previous view
         *
         */
        deletePointsView.prototype.back = function () {
            baja.outln("inside: deletePointsView.prototype.back ...");
            return new promise(function (resolve) {
                // do any cleanup/backend communication needed here before moving to previous screen.
                //resolve once all the input is validated and good to move to next screen
                $(dom).find(".content-section").css("display", "block");
                resolve(true);
            });
        };

        /**
         * cancel operation - cancel button supported cancellable operation in current view
         */
        deletePointsView.prototype.cancel = function () {
            /*
             * if cancel button is available, the action should be implemented here.
            */
        };

        //these are prototype functions and needs to be overriden in extended screens.
        deletePointsView.prototype.updateView = function () {
            throw new Error('Cannot call abstract method, please overwrite in current view');
        };

        function completeJob(job) {
            return new Promise(function (resolve, reject) {
                function checkIfJobIsComplete() {
                    switch (job.get('jobState').getTag()) {
                        case 'success':
                            return resolve();
                        case 'failed':
                            deletePointsView.updateActionButtonsStatus(
                                $(dom).parent(),
                                {
                                    disabledBack: false,
                                    disabledNext: false
                                }
                            );
                            return reject();

                        default :
                            break;
                    }
                }

                var sub = new baja.Subscriber();
                sub.attach('changed', checkIfJobIsComplete);
                sub.subscribe(job).then(checkIfJobIsComplete);
            });
        }

        function notifyUser(message) {
            baja.outln(message);
            //hide progress.
            $(dom).find(".operation-progress").removeClass("show-progress");
            $(dom).find(".content-section").css("display", "none");
            $(dom).find("#PDM-step5-deletePointsView .error-message").html(message);
        }

        return deletePointsView;
    });
