/*
 *
 *  * Copyright (c) 2019. Honeywell International, Inc. All Rights Reserved.
 *    created by H148225 on 7/18/2019
 *
 */

define(['baja!',
        'jquery',
        'underscore',
        'Promise',
        'nmodule/easyDatabaseManager/rc/js/defaultView',
        'hbs!nmodule/easyDatabaseManager/rc/hbs/unusedPointsViewTemplate',
        'lex!easyDatabaseManager',
        'css!nmodule/easyDatabaseManager/rc/css/pointDataManager',
        'css!nmodule/easyDatabaseManager/rc/css/datatable'],
    function (baja,
              $,
              _,
              promise,
              defaultView,
              template,
              lexicons) {

        'use strict';

        var dom,
            isUnusedSkiped = false,
            lex = lexicons[0], isIncreaseFooterSize = true;


        // unusedPointsView step
        // arguments: dom, vname, id, message, summaryMessage, backBttnMsg, nextBttnMsg, cancelBttnMsg
        var unusedPointsView = function () {
            defaultView.apply(this, arguments);

            //update which buttons will be available
            this.buttonstatus = {
                disabledBack: false,
                disabledNext: false,
                disabledSkip: false
            };
        };
        unusedPointsView.prototype = Object.create(defaultView.prototype,
            {'constructor': unusedPointsView}
        );

        //these variables will be updated in loadView with values passed from PointDatabaseManagerWidget
        unusedPointsView.component = null;
        unusedPointsView.service = null;
        unusedPointsView.subscriber = null;
        // call this function anytime you want to update nav action button status to enable/disable them
        unusedPointsView.updateActionButtonsStatus = null;

        /**
         * Load view of current wizard step - unusedPointsView
         * @param element: dom element the view will be appended to.
         * @param params: javascript object with widget parameters required to load the view within the widget
         */
        unusedPointsView.prototype.loadView = function (element, params) {
            var that = this;
            dom = element;
            unusedPointsView.component = params.component;
            unusedPointsView.service = params.service;
            unusedPointsView.subscriber = params.subscriber;
            unusedPointsView.updateActionButtonsStatus = params.updateActionButtonsStatus;
            isUnusedSkiped = false;
            isIncreaseFooterSize = params.pdmData.isIncreaseFooterSize;

            $(element).append(template({
                    vid: this.params.vid,
                    summaryMessage: this.params.summaryMessage,
                    stepCount: this.params.stepCount,
                    stepDescription: this.params.stepDescription

                }
            ));
            $(element).find(".PDM-contents-step-summary").css("border-top-color",params.colorCode);
            that.$getUnusedSearchCriteria();

            //remove margin increased set in the first screen since content size less from screen 2-4
            $(".PDM-footer").removeClass("PDM-footer-increase-margin");

            if(isUnusedSkiped) {
                unusedPointsView.prototype.updateActionButtonsStatus(
                    $(dom).parent(),
                    {
                        disabledBack: false,
                        disabledNext: false,
                        disabledSkip: true
                    }
                );
            }
        };

        unusedPointsView.prototype.showLoadedView = function () {
            if (isUnusedSkiped) {
                unusedPointsView.prototype.updateActionButtonsStatus(
                    $(dom).parent(),
                    {
                        disabledBack: false,
                        disabledNext: false,
                        disabledSkip: true
                    }
                );
            }
        };
            /**
         * validate user input before proceeding to next screen.
         * This is called before view.prototype.next is called.
         *
         * @returns {boolean}: return true/false based on validity of user input
         */
        unusedPointsView.prototype.validate = function () {
            baja.outln("inside: unusedPointsView.prototype.validate ...");
            return new promise(function (resolve) {
                //do any validations needed here.
                //resolve once all the input is validated and good to move to next screen
                resolve(true);
            });
        };

        /**
         * clean up and save before going to next view
         *
         */
        unusedPointsView.prototype.next = function () {
            var that = this;
            baja.outln("inside: unusedPointsView.prototype.next ...");

            var criteria = {};

            // If any checkbox is selected, set Enabled = true;
            // var item = {};
            criteria["enabled"] = "true";

            $.each($(that.dom).find(".ui-state-default"), function () {
                criteria[$(this).attr('id')] = $(this).hasClass("ui-state-active") ? "true" : "false";
            });

            return baja.rpc({
                typeSpec: 'easyDatabaseManager:PointDataManagerRpc',
                method: 'saveUnusedSearchCriteria',
                args: [
                    {
                        enabled: criteria["enabled"],
                        includePxView: criteria["includePxView"],
                        includeTagBasedPxView: criteria["includeTagBasedPxView"],
                        includeLinkMark: criteria["includeLinkMark"],
                        includeAlarmExt: criteria["includeAlarmExt"],
                        includeHistoryExt: criteria["includeHistoryExt"],
                        includeHistoryImport: criteria["includeHistoryImport"],
                        includePointListView: criteria["includePointListView"],
                        includeAnalytics: criteria["includeAnalytics"],
                        includeLinkTrendLog: criteria["includeLinkTrendLog"] === undefined ? false : criteria["includeLinkTrendLog"],
                        includeEasyTemplate: criteria["includeEasyTemplate"]
                    }
                ]
            }).then(function () {
                return new promise(function (resolve) {
                    // do any backend communication needed here before moving to next screen.
                    //resolve once all the input is validated and good to move to next screen

                    that.data = {unusedPointsViewData: {isUnusedSkiped: false}};

                    resolve(true);
                });
            })
        };

        /**
         * clean up and save before going to previous view
         *
         */
        unusedPointsView.prototype.back = function () {
            baja.outln("inside: unusedPointsView.prototype.back ...");

            //set footer size more for screen 1
            if(isIncreaseFooterSize)
                $(".PDM-footer").addClass("PDM-footer-increase-margin");
            return new promise(function (resolve) {
                // do any cleanup/backend communication needed here before moving to previous screen.
                //resolve once all the input is validated and good to move to next screen
                resolve(true);
            });
        };

        /**
         * cancel operation - cancel button supported cancellable operation in current view
         *
         */
        unusedPointsView.prototype.cancel = function() {
            /*
             * if cancel button is available, the action should be implemented here.
            */
        };

        /**
         * clean up and save before skipping to next view
         *
         */
        unusedPointsView.prototype.skip = function () {
            var that = this;
            baja.outln("inside: unusedPointsView.prototype.skip ...");

            return baja.rpc({
                typeSpec: 'easyDatabaseManager:PointDataManagerRpc',
                method: 'saveUnusedSearchCriteria',
                args: [
                    {
                        enabled: false,
                        includePxView: false,
                        includeTagBasedPxView: false,
                        includeLinkMark: false,
                        includeAlarmExt: false,
                        includeHistoryExt: false,
                        includeHistoryImport: false,
                        includePointListView: false,
                        includeLinkTrendLog: false,
                        includeEasyTemplate: false
                    }
                ]
            }).then(function () {
                return new promise(function (resolve) {
                    // do any backend communication needed here before moving to next screen.
                    //resolve once all the input is validated and good to move to next screen

                    that.data = {unusedPointsViewData: {isUnusedSkiped: true}};

                    resolve(true);
                });
            })
        };


        //these are prototype functions and needs to be overriden in extended screens.
        unusedPointsView.prototype.updateView = function () {
            throw new Error('Cannot call abstract method, please overwrite in current view');
        };

        // getList
        unusedPointsView.prototype.$getUnusedSearchCriteria = function () {
            var that = this;
            baja.rpc({
                typeSpec: 'easyDatabaseManager:PointDataManagerRpc',
                method: 'getUnusedSearchCriteria'
            }).then(function (list) {
                that.$displayCriteria(list);
            })
        };

        var _addEventListeners = function (domobj) {
            //select or unselect individual rows
            $(domobj).find("div.ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default").on('click',
                function (event) {
                    var $target = $(event.target);
                    if ($target.hasClass("ui-chkbox-icon")) {
                        $target.parent().toggleClass("ui-state-active");
                        $target.toggleClass("pi pi-check");
                    } else {
                        $target.toggleClass("ui-state-active");
                        $target.find(".ui-chkbox-icon.ui-clickable").toggleClass("pi pi-check");
                    }
                });

        };

        // getList
        unusedPointsView.prototype.$displayCriteria = function (list) {
            var that = this,
                criteriaList = JSON.parse(list);

            var checkboxContainer = that.dom.find('#PDM-step2-unused-points-checkbox-container');
            var body = checkboxContainer.find('.ui-datatable-data');

            _.each(criteriaList, function (item) {

                if (item.PropertyName !== "enabled") {

                    // Column 1 Text
                    var row = $('<tr class="ui-widget-content"></tr>'),
                        tdText = $('<td></td>'),
                        span = $('<span class=ui-cell-data></span>'),
                        div = $('<div class="cell-data-wrap"></div>');

                    $(div).text(lex.get("pdm.PointDatabaseManager.UnusedPoints.criteria."+item.PropertyName));
                    span.append(div);
                    tdText.append(span);
                    row.append(tdText);

                    //Column 2 Value
                    var tdCheckBox = $('<td class="ui-selection-column ui-selection-column-criteria"></td>'),
                        spanCheckBox = $('<span class=ui-cell-data></span>'),
                        checkboxWidget = $('<div class="ui-chkbox ui-widget"></div>'),
                        inputDiv = $('<div class="ui-helper-hidden-accessible"></div>'),
                        inputCheckbox = $('<input type="checkbox"></input>'),
                        cellCSSCheckBox = $('<div class="ui-chkbox-box ui-widget ui-corner-all ui-state-default"></div>'),
                        cellCSSCheckSpan = $('<span class="ui-chkbox-icon ui-clickable"></span>');
                    if (item.PropertyValue === 'true') {
                        cellCSSCheckBox = $('<div class="ui-chkbox-box ui-widget ui-corner-all ui-state-default ui-state-active"></div>');
                        cellCSSCheckSpan = $('<span class="ui-chkbox-icon ui-clickable pi pi-check"></span>');
                    }

                    $(cellCSSCheckBox).attr('id', item.PropertyName);

                    inputDiv.append(inputCheckbox);
                    checkboxWidget.append(inputDiv);
                    cellCSSCheckBox.append(cellCSSCheckSpan);
                    checkboxWidget.append(cellCSSCheckBox);
                    spanCheckBox.append(checkboxWidget);
                    tdCheckBox.append(spanCheckBox);

                    row.append(tdCheckBox);
                    body.append(row);
                }
            });
            _addEventListeners(checkboxContainer);
        };

        return unusedPointsView;
    });
