/*
 *
 *  * Copyright (c) 2019. Honeywell International, Inc. All Rights Reserved.
 *    created by H148225 on 7/19/2019
 *
 */

define(['baja!',
        'jquery',
        'underscore',
        'Promise',
        'dialogs',
        'nmodule/easyDatabaseManager/rc/js/defaultView',
        'hbs!nmodule/easyDatabaseManager/rc/hbs/unwantedPointsViewTemplate',
        'lex!easyDatabaseManager',
        'css!nmodule/easyDatabaseManager/rc/css/pointDataManager',
        'css!nmodule/easyDatabaseManager/rc/css/datatable'],
    function (baja,
              $,
              _,
              promise,
              dialogs,
              defaultView,
              template,
              lexicons) {

        'use strict';

        //lexicons
        var lex = lexicons[0],
            labelSearchCount = 1,
            dom,
            domLoadedView,
            textboxLabelPlaceHolder=lex.get("pdm.PointDatabaseManager.stepThree.labelTextBox.placeholder"),
            isUnusedSkiped, savedCriteria, hasSavedCriteria=false;

        // unwantedPointsView step
        // arguments: dom, vname, id, message, summaryMessage, backBttnMsg, nextBttnMsg, cancelBttnMsg
        var unwantedPointsView = function () {
            defaultView.apply(this, arguments);

            //update which buttons will be available
            this.buttonstatus = {
                disabledBack: false,
                disabledNext: false,
                disabledSkip: false
            };
        };
        unwantedPointsView.prototype = Object.create(defaultView.prototype,
            {'constructor': unwantedPointsView}
        );

        //these variables will be updated in loadView with values passed from PointDatabaseManagerWidget
        unwantedPointsView.component = null;
        unwantedPointsView.service = null;
        unwantedPointsView.subscriber = null;
        // call this function anytime you want to update nav action button status to enable/disable them
        unwantedPointsView.updateActionButtonsStatus = null;

        /**
         * Load view of current wizard step - unwantedPointsView
         * @param element: dom element the view will be appended to.
         * @param params: javascript object with widget parameters required to load the view within the widget
         */
        unwantedPointsView.prototype.loadView = function (element, params) {
            var that = this;
            domLoadedView = element;
            unwantedPointsView.component = params.component;
            unwantedPointsView.service = params.service;
            unwantedPointsView.subscriber = params.subscriber;
            unwantedPointsView.updateActionButtonsStatus = params.updateActionButtonsStatus;
            isUnusedSkiped = params.pdmData.unusedPointsViewData.isUnusedSkiped;
            labelSearchCount = 1;

            if (isUnusedSkiped) {

                unwantedPointsView.updateActionButtonsStatus(
                    $(element).parent(),
                    {
                        disabledBack: false,
                        disabledNext: false,
                        disabledSkip: true
                    }
                );

            }
            $(element).append(template({
                    vid: this.params.vid,
                    summaryMessage: this.params.summaryMessage,
                    stepCount: this.params.stepCount,
                    stepDescription: this.params.stepDescription,
                    note:lex.get("pdm.PointDatabaseManager.stepThreeNote"),
                    note1:lex.get("pdm.PointDatabaseManager.stepThreeNote1"),
                    labelSearchCriteriaText: lex.get("pdm.PointDatabaseManager.unwantedPoints.labelSearchCriteria"),
                    newLabelSearchText: lex.get("pdm.PointDatabaseManager.newSearchLabel"),
                    loadCriteraButtonText: lex.get("pdm.PointDatabaseManager.stepThree.loadCriteriaButtonText"),
                    include: lex.get("pdm.PointDatabaseManager.UnwantedPoints.criteria.Include"),
                    textboxLabelPlaceHolder: textboxLabelPlaceHolder
            }
            ));


            $(element).find(".PDM-contents-step-summary").css("border-top-color",params.colorCode);
            $(element).find(".addLabelSearchIcon").unbind('click').click(function () {

                if (labelSearchCount >= 10) {

                    var dialog = dialogs.make({
                        title: lex.get("pdm.PointDatabaseManager.unwantedPoints.error.title"),
                        content: lex.get("pdm.PointDatabaseManager.unwantedPoints.error.default.label.limit"),
                        ok: function (dlg) {
                            var $dialog = dlg.jq();
                            $dialog.find("div.js-dialog.ux-root").removeClass("confirm-js-dialog");
                            dlg.close();
                        }
                    });
                    dialog.show();
                    return;
                }
                var checkboxContainer = $(element).find('#PDM-step3-Label-Search-Criteria');
                var tBody = checkboxContainer.find('.ui-datatable-data');
                var row =getDynamicTextBox('');

                tBody.append(row);
                labelSearchCount = labelSearchCount + 1;
            });
            that.$getUnwantedSearchCriteria();
            getSavedCriteria(element);

            $(element).find(".load-criteria").unbind('click').click(function () {
                loadCriteria(savedCriteria, element);
            });

        };

        unwantedPointsView.prototype.showLoadedView = function () {
            getSavedCriteria(domLoadedView);
            if (isUnusedSkiped) {
                unwantedPointsView.updateActionButtonsStatus(
                    $(domLoadedView).parent(),
                    {
                        disabledBack: false,
                        disabledNext: false,
                        disabledSkip: true
                    }
                );
            }
        };

        /**
         * validate user input before proceeding to next screen.
         * This is called before view.prototype.next is called.
         *
         * @returns {boolean}: return true/false based on validity of user input
         */
        unwantedPointsView.prototype.validate = function () {
            baja.outln("inside: unwantedPointsView.prototype.validate ...");
            return new promise(function (resolve) {
                //do any validations needed here.
                //resolve once all the input is validated and good to move to next screen
                resolve(true);
            });
        };

        /**
         * clean up and save before going to next view
         *
         */
        unwantedPointsView.prototype.next = function () {
            var that = this,
                dialog,
                checkboxContainer = that.dom.find('#PDM-step3-unwanted-points-checkbox-container');
            baja.outln("inside: unwantedPointsView.prototype.next ...");
            var isMinimumSelected = false;
            $.each($(checkboxContainer).find(".ui-state-default:lt(3)"), function () {
                if ($(this).hasClass("ui-state-active")) {
                    isMinimumSelected = true;
                }
            });

            var defaultLabelSearch = that.dom.find("#PDM-step3-default-label-search");
            return new Promise(function (resolve) {

                if (!isMinimumSelected) {

                    // Any of the first 3 should be selected
                    dialog = dialogs.make({
                        title: lex.get("pdm.PointDatabaseManager.unwantedPoints.error.title"),
                        content: lex.get("pdm.PointDatabaseManager.unwantedPoints.error.message",
                            lex.get("pdm.PointDatabaseManager.UnwantedPoints.criteria.points"),
                            lex.get("pdm.PointDatabaseManager.UnwantedPoints.criteria.schedules"),
                            lex.get("pdm.PointDatabaseManager.UnwantedPoints.criteria.histories")),
                        ok: function (dlg) {
                            var $dialog = dlg.jq();
                            $dialog.find("div.js-dialog.ux-root").removeClass("confirm-js-dialog");
                            dlg.close();
                        }
                    });
                    dialog.show();
                    resolve(false);
                } else if ($(defaultLabelSearch).val() === "" && $(defaultLabelSearch).hasClass("PDM-step3-label-search")) {
                        // At least one Label should be selected
                        dialog = dialogs.make({
                            title: lex.get("pdm.PointDatabaseManager.unwantedPoints.error.title"),
                            content: lex.get("pdm.PointDatabaseManager.unwantedPoints.error.default.label.search.empty") + ".",
                            ok: function (dlg) {
                                var $dialog = dlg.jq();
                                $dialog.find("div.js-dialog.ux-root").removeClass("confirm-js-dialog");
                                dlg.close();
                            }
                        });
                        dialog.show();
                        resolve(false);
                } else {
                    // All the textboxes should have some value
                    var isTextboxesEmpty = false;
                    // % or $ should not be in textboxes
                    var isSpecialChar = false;
                    var labels = [];
                    var labelSearchCriteria = that.dom.find("#PDM-step3-Label-Search-Criteria");
                    $.each($(labelSearchCriteria).find(":input"), function () {
                        var temp = $(this).val();
                        if (temp === "" && $(this).hasClass("PDM-step3-label-search")) {
                            isTextboxesEmpty = true;
                            return;
                        }
                        else if (
                            !(temp.indexOf("$") === -1
                                && temp.indexOf("%") === -1
                                && temp.indexOf("'") === -1
                                && $(this).hasClass("PDM-step3-label-search"))) {
                            isSpecialChar = true;
                            return;
                        } else {
                            labels.push($(this).val());
                        }
                    });

                    if (isTextboxesEmpty) {
                        dialog = dialogs.make({
                            title: lex.get("pdm.PointDatabaseManager.unwantedPoints.error.title"),
                            content: lex.get("pdm.PointDatabaseManager.unwantedPoints.error.default.label.empty"),
                            ok: function (dlg) {
                                var $dialog = dlg.jq();
                                $dialog.find("div.js-dialog.ux-root").removeClass("confirm-js-dialog");
                                dlg.close();
                            }
                        });
                        dialog.show();
                        resolve(false);
                    } else if (isSpecialChar) {
                        dialog = dialogs.make({
                            title: lex.get("pdm.PointDatabaseManager.unwantedPoints.error.title"),
                            content: lex.get("pdm.PointDatabaseManager.unwantedPoints.error.default.label.specialChar"),
                            ok: function (dlg) {
                                var $dialog = dlg.jq();
                                $dialog.find("div.js-dialog.ux-root").removeClass("confirm-js-dialog");
                                dlg.close();
                            }
                        });
                        dialog.show();
                        resolve(false);
                    }
                    else {
                        // If any checkbox is selected, set Enabled = true;
                        var criteria = {};
                        criteria["enabled"] = "true";
                        $.each($(checkboxContainer).find(".ui-state-default"), function () {
                            var cri = ($(this).attr('id')).replace("chk", "");
                            criteria[cri] = $(this).hasClass("ui-state-active") ? "true" : "false";
                        });

                        dialog = dialogs.make({
                            title: lex.get("pdm.PointDatabaseManager.unwantedPointsViewMsg"),
                            content: lex.get("pdm.PointDatabaseManager.unwantedPoints.save.criteria.save"),
                            yes: function (dlg) {
                                var $dialog = dlg.jq();
                                $dialog.find("div.js-dialog.ux-root").removeClass("confirm-js-dialog");
                                dlg.close();
                                baja.rpc({
                                    typeSpec: 'easyDatabaseManager:PointDataManagerRpc',
                                    method: 'saveUnwantedSearchCriteria',
                                    args: [{
                                        enabled: criteria["enabled"],
                                        points:  criteria["points"],
                                        schedules:  criteria["schedules"],
                                        histories:  criteria["histories"],
                                        includeLinkMark:  criteria["includeLinkMark"],
                                        includeAnalytics: criteria["includeAnalytics"],
                                        includePxView:  criteria["includePxView"],
                                        includeTagBasedPxView:  criteria["includeTagBasedPxView"],
                                        includeHistoryImportAnalytics:  criteria["includeHistoryImportAnalytics"],
                                        deleteLinkedHistories: criteria["deleteLinkedHistories"] === undefined ? false : criteria["deleteLinkedHistories"],

                                        saveRequired: true,
                                        labels: labels
                                    }]
                                }).then(function () {
                                    that.data = {
                                        unwantedPointsViewData: {
                                            isUnwantedSkiped: false,
                                            labelSearchCriteriaList: labels
                                        }
                                    };
                                    resolve(true);
                                });
                            },
                            no: function (dlg) {
                                var $dialog = dlg.jq();
                                $dialog.find("div.js-dialog.ux-root").removeClass("confirm-js-dialog");
                                dlg.close();
                                baja.rpc({
                                    typeSpec: 'easyDatabaseManager:PointDataManagerRpc',
                                    method: 'saveUnwantedSearchCriteria',
                                    args: [{
                                        enabled: criteria["enabled"],
                                        points:  criteria["points"],
                                        schedules:  criteria["schedules"],
                                        histories:  criteria["histories"],
                                        includeLinkMark:  criteria["includeLinkMark"],
                                        includeAnalytics: criteria["includeAnalytics"],
                                        includePxView:  criteria["includePxView"],
                                        includeTagBasedPxView:  criteria["includeTagBasedPxView"],
                                        includeHistoryImportAnalytics:  criteria["includeHistoryImportAnalytics"],
                                        deleteLinkedHistories: criteria["deleteLinkedHistories"] === undefined ? false : criteria["deleteLinkedHistories"],
                                        saveRequired: false,
                                        labels: labels
                                    }]
                                }).then(function () {
                                    that.data = {
                                        unwantedPointsViewData: {
                                            isUnwantedSkiped: false,
                                            labelSearchCriteriaList: labels
                                        }
                                    };
                                    resolve(true);
                                });
                            }
                        });
                        dialog.show();
                    }
                }
            });
        };

        /**
         * clean up and save before skipping to next view
         *
         */
        unwantedPointsView.prototype.skip = function () {
            var that = this;
            baja.outln("inside: unwantedPointsView.prototype.skip ...");

            if (isUnusedSkiped) {

                unwantedPointsView.updateActionButtonsStatus(
                    $(that.dom).parent(),
                    {
                        disabledBack: false,
                        disabledNext: false,
                        disabledSkip: true
                    }
                );

                return new promise(function (resolve) {
                    //do any validations needed here.
                    //resolve once all the input is validated and good to move to next screen
                    resolve(false);
                });
            }

            var criteria = [];

            // If any checkbox is selected, set Enabled = true;
            var item = {};
            item["PropertyName"] = "enabled";
            item["PropertyValue"] = "false";
            item["labels"]=[];
            criteria.push(item);
            var checkboxContainer = that.dom.find('#PDM-step3-unwanted-points-checkbox-container');
            $.each($(checkboxContainer).find(".ui-state-default"), function () {
                item = {};
                item["PropertyName"] = $(this).attr('id');
                item["PropertyValue"] = "false";
                criteria.push(item);
            });

            return baja.rpc({
                typeSpec: 'easyDatabaseManager:PointDataManagerRpc',
                method: 'saveUnwantedSearchCriteria',
                args: [{
                    enabled: false,
                    points: false,
                    schedules: false,
                    histories: false,
                    includeLinkMark: false,
                    includePxView: false,
                    includeTagBasedPxView: false,
                    deleteLinkedHistories: false,
                    saveRequired: false,
                    labels: []
                }]
            }).then(function () {
                return new promise(function (resolve) {
                    // do any backend communication needed here before moving to next screen.
                    //resolve once all the input is validated and good to move to next screen

                    that.data = {
                        unwantedPointsViewData: {
                            isUnwantedSkiped: true,
                            labelSearchCriteriaList: []
                        }
                    };

                    resolve(true);
                });
            })
        };
        /**
         * clean up and save before going to previous view
         *
         */
        unwantedPointsView.prototype.back = function () {
            var that = this;
            baja.outln("inside: unwantedPointsView.prototype.back ...");
            return new promise(function (resolve) {
                // do any cleanup/backend communication needed here before moving to previous screen.
                //resolve once all the input is validated and good to move to next screen

                // On Back we must reset skip from Unused Points
                that.data = {unusedPointsViewData: {isUnusedSkiped: false}};

                resolve(true);
            });
        };

        /**
         * cancel operation - cancel button supported cancellable operation in current view
         * This is invoked only if this.params.isCancelAvailable is set to true.
         */
        unwantedPointsView.prototype.cancel = function () {
            /*
             * if cancel button is available, the action should be implemented here.
            */
        };

        //these are prototype functions and needs to be overriden in extended screens.
        unwantedPointsView.prototype.updateView = function () {
            throw new Error('Cannot call abstract method, please overwrite in current view');
        };

        // getList
        unwantedPointsView.prototype.$getUnwantedSearchCriteria = function () {
            var that = this;
            baja.rpc({
                typeSpec: 'easyDatabaseManager:PointDataManagerRpc',
                method: 'getUnwantedSearchCriteria'
            }).then(function (list) {
                that.$displayCriteria(list);
            })
        };


        var _addEventListeners = function (domObj) {
            //select or unselect individual rows
            $(domObj).find("div.ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default").on('click',
                function (event) {
                    var $target = $(event.target);
                    if ($target.hasClass("ui-chkbox-icon")) {
                            $target.parent().toggleClass("ui-state-active");
                            $target.toggleClass("pi pi-check");
                            if ($target.parent().hasClass("points")) {
                                domLoadedView.find("#deleteLinkedHistories .cell-data-wrap").css("color", "#808080d9");
                                domLoadedView.find("#deleteLinkedHistories .ui-state-default").removeClass("ui-state-active");
                                domLoadedView.find("#deleteLinkedHistories .ui-chkbox-icon.ui-clickable").removeClass("pi pi-check");
                                domLoadedView.find("#deleteLinkedHistories .ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default.deleteLinkedHistories").css("border-color","#d0d0d0");
                            }
                    } else {
                        if($target.hasClass("deleteLinkedHistories")){
                            if(domLoadedView.find("#points .ui-state-active").length > 0)
                            {
                                $target.toggleClass("ui-state-active");
                                $target.find(".ui-chkbox-icon.ui-clickable").toggleClass("pi pi-check");
                            }
                        }else {
                            $target.toggleClass("ui-state-active");
                            $target.find(".ui-chkbox-icon.ui-clickable").toggleClass("pi pi-check");
                            if ($target.hasClass("points")) {
                                domLoadedView.find("#deleteLinkedHistories .cell-data-wrap").css("color", "black");
                                domLoadedView.find("#deleteLinkedHistories .ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default.deleteLinkedHistories").css("border-color","black");
                            }
                        }
                    }

                });
            // select or unselect all rows
        };


        // showList
        unwantedPointsView.prototype.$displayCriteria = function (list) {
            var that = this,
                criteriaList = JSON.parse(list);

            var checkboxContainer = that.dom.find('#PDM-step3-unwanted-points-checkbox-container');
            var body = checkboxContainer.find('.ui-datatable-data');
            var flg = false;

            _.each(criteriaList, function (item) {

                if (item.PropertyName !== "enabled") {

                    // Column 2 Text
                    var row = $('<tr class="ui-widget-content tr-even" id="' + item.PropertyName.replace(/\s/g, '') + '"></tr>'),
                        tdText = $('<td></td>'),
                        span,
                        div = $('<div class="cell-data-wrap"></div>');

                    //make flex container for the sub option
                    if(item.PropertyName == "deleteLinkedHistories" || item.PropertyName == "includeHistoryImportAnalytics")
                    {
                        span = $('<div class=ui-cell-data></div>')
                    }
                    else
                    {
                        span = $('<span class=ui-cell-data></span>')
                    }
                    if(item.PropertyName==="deleteLinkedHistories") {
                        row.addClass("hide-linked-histories");
                    }
                    $(div).text(lex.get("pdm.PointDatabaseManager.UnwantedPoints.criteria."+item.PropertyName));



                    //Column 1 Value
                    var tdCheckBox,
                        emptycell = $('<td></td>'),
                        spanCheckBox = $('<span class=ui-cell-data></span>'),
                        checkboxWidget = $('<div class="ui-chkbox ui-widget"></div>'),
                        inputDiv = $('<div class="ui-helper-hidden-accessible"></div>'),
                        inputCheckbox = $('<input type="checkbox"></input>'),
                        cellCSSCheckBox = $('<div class="ui-chkbox-box ui-widget ui-corner-all ui-state-default"></div>'),
                        cellCSSCheckSpan = $('<span class="ui-chkbox-icon ui-clickable"></span>');


                    if (item.PropertyValue === 'true') {
                        cellCSSCheckBox.addClass("ui-state-active");
                        cellCSSCheckSpan.addClass("pi pi-check");

                        if(item.PropertyName==="points") {
                            flg = true;
                        }
                    }
                    if(item.PropertyName==="points")
                    {
                        cellCSSCheckBox.addClass("points");
                    }
                    if(item.PropertyName==="deleteLinkedHistories")
                    {
                        cellCSSCheckBox.addClass("deleteLinkedHistories");
                    }

                    $(cellCSSCheckBox).attr('id', "chk"+ item.PropertyName);

                    inputDiv.append(inputCheckbox);
                    checkboxWidget.append(inputDiv);
                    cellCSSCheckBox.append(cellCSSCheckSpan);
                    checkboxWidget.append(cellCSSCheckBox);
                    spanCheckBox.append(checkboxWidget);

                    if(item.PropertyName == "deleteLinkedHistories" || item.PropertyName == "includeHistoryImportAnalytics")
                    {
                        tdCheckBox = $('<div class="ui-selection-column ui-selection-column-criteria ui-selection-column-aligncenter ui-selection-column-step3"></div>');
                        tdCheckBox.append(spanCheckBox);
                        row.append(emptycell);
                        span.append(tdCheckBox);

                    }
                    else
                    {
                        tdCheckBox = $('<td class="ui-selection-column ui-selection-column-criteria ui-selection-column-aligncenter ui-selection-column-step3"></td>');
                        tdCheckBox.append(spanCheckBox);
                        row.append(tdCheckBox);
                    }


                    span.append(div);
                    tdText.append(span);

                    row.append(tdText);
                    body.append(row);
                }
            });
            _addEventListeners(checkboxContainer);
            if(flg) {
                domLoadedView.find("#deleteLinkedHistories .cell-data-wrap").css("color","black");
                domLoadedView.find("#deleteLinkedHistories .ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default.deleteLinkedHistories").css("border-color","black");
            }


        };
        function getSavedCriteria(element) {
            baja.rpc({
                typeSpec: 'easyDatabaseManager:PointDataManagerRpc',
                method: 'loadUnwantedSearchCriteria'

            }).then(function (output) {
               if (output !== "{}" )
                {
                    savedCriteria = output;
                    hasSavedCriteria=true;
                }
                else
                {
                    hasSavedCriteria=false;
                }
                if(hasSavedCriteria){
                    $(element).find("#load-criteria").prop('disabled', false);
                }
                else
                {
                    $(element).find("#load-criteria").prop('disabled', true);
                }
            });

        };

        function loadCriteria(data, element) {
            labelSearchCount = 1;
            var jsonObj=JSON.parse(data);
            var savedLabels=jsonObj.unwantedPointSearchCriteria.labelEntities;
            $(element).find(".dynamic-text-box").remove();
            var firstLabel=savedLabels[0].name;
            var isFirstLabel=true;
            var defaultTextBox = $(element).find('#PDM-step3-default-label-search').replaceWith('<input type="text" id="PDM-step3-default-label-search" class="PDM-step3-label-search" maxlength="256" value="'+firstLabel+'" />');
            var textboxContainer = $(element).find('#PDM-step3-Label-Search-Criteria');
            var tBody = textboxContainer.find('.ui-datatable-data');
            _.each(savedLabels, function (label) {
                if(isFirstLabel)
                {
                    isFirstLabel=false;
                    return;
                }
                var row = getDynamicTextBox(label.name);
                tBody.append(row);
                labelSearchCount = labelSearchCount + 1;
            });


            var savedCriteriaJson=jsonObj.unwantedPointSearchCriteria.searchOptionsEntity;
            var checkboxContainer = $(element).find('#PDM-step3-unwanted-points-checkbox-container');

            for (var key in savedCriteriaJson) {
                if (savedCriteriaJson.hasOwnProperty(key)) {
                    var checkbox = $(checkboxContainer).find("#chk"+key);
                    if("Y" === savedCriteriaJson[key])
                    {
                        $(checkbox).addClass("ui-state-active")
                        $(checkbox).children("span").addClass("pi pi-check");

                        // If it is points
                        if(key === "points") {
                            domLoadedView.find("#deleteLinkedHistories .cell-data-wrap").css("color","black");
                            domLoadedView.find("#deleteLinkedHistories .ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default.deleteLinkedHistories").css("border-color","black");
                        }
                    }
                    else {
                        if(key === "points") {
                            domLoadedView.find("#deleteLinkedHistories .cell-data-wrap").css("color","#808080d9");
                            domLoadedView.find("#deleteLinkedHistories .ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default.deleteLinkedHistories").css("border-color","#d0d0d0");
                        }
                        $(checkbox).removeClass("ui-state-active")
                        $(checkbox).children("span").removeClass("pi pi-check");
                    }
                }
            }
        };
        // get dynamic text box
        function getDynamicTextBox(label) {

            // Column 1 Text
            var row = $('<tr class="ui-widget-content dynamic-text-box" ></tr>'),
                tdText = $('<td></td>'),
                span = $('<span class=ui-cell-data></span>'),
                div = $('<div class="cell-data-wrap"><input type="text" class="PDM-step3-label-search" maxlength="256" value="'+label+'" placeholder="'+textboxLabelPlaceHolder+'"></input></div>');

            span.append(div);
            tdText.append(span);
            row.append(tdText);

            //Column 2 Value
            var tdTextBox = $('<td class="ui-selection-column ui-selection-column-criteria"></td>'),
                spanTextBox = $('<span class=ui-cell-data></span>'),
                textBoxWidget = $('<div class="ui-chkbox ui-widget"></div>'),
                cellCSSTextBox = $('<div class="ui-chkbox-box ui-widget ui-corner-all ui-state-default"></div>'),
                cellWrapper = $('<div class="delete-icon-wrapper"></div>'),
                cellCSSTextSpan = $('<image class="deleteLabelSearchIcon" src="/module/easyDatabaseManager/rc/images/delete.jpg"></image>');

            cellCSSTextSpan.unbind('click').click(function () {
                $(this).closest('tr').remove();
                labelSearchCount = labelSearchCount - 1;
            });

            cellWrapper.append(cellCSSTextSpan);
            cellCSSTextBox.append(cellWrapper);
            textBoxWidget.append(cellCSSTextBox);
            spanTextBox.append(textBoxWidget);
            tdTextBox.append(spanTextBox);
            row.append(tdTextBox);
            return row;

        };

        return unwantedPointsView;
    });
